package awais.instagrabber.utils.dialogs;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;

import awais.instagrabber.R;
import awais.instagrabber.activities.Main;
import awais.instagrabber.adapters.SimpleAdapter;
import awais.instagrabber.utils.DataBox;
import awais.instagrabber.utils.Utils;

import static awais.instagrabber.utils.Utils.sharedPreferences;

public class QuickAccessDialog extends BottomSheetDialogFragment implements DialogInterface.OnShowListener {
    private Activity activity;
    private boolean cookieChanged;
    private String userQuery;
    private SimpleAdapter<DataBox.FavoriteModel> favoritesAdapter;

    public QuickAccessDialog setQuery(final String userQuery) {
        this.userQuery = userQuery;
        return this;
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        final Dialog dialog = super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener(this);

        final Context context = getContext();
        activity = context instanceof Activity ? (Activity) context : getActivity();

        final View contentView = View.inflate(activity, R.layout.quick_access_dialog, null);

        final View btnFavorite = contentView.findViewById(R.id.btnFavorite);
        final boolean isQuery = !Utils.isEmpty(userQuery);
        btnFavorite.setVisibility(isQuery ? View.VISIBLE : View.GONE);

        final View.OnClickListener onClickListener = v -> {
            final Object tag = v.getTag();
            if (btnFavorite == v) {
                if (isQuery) {
                    Utils.dataBox.addFavorite(new DataBox.FavoriteModel(userQuery, System.currentTimeMillis()));
                    favoritesAdapter.setItems(Utils.dataBox.getAllFavorites());
                }

            } else if (tag instanceof DataBox.FavoriteModel) {
                if (Main.scanHack != null) {
                    Main.scanHack.onResult(((DataBox.FavoriteModel) tag).getQuery());
                    dismiss();
                }

            } else if (tag instanceof DataBox.CookieModel) {
                final DataBox.CookieModel cookieModel = (DataBox.CookieModel) tag;
                if (!cookieModel.isSelected()) {
                    sharedPreferences.edit().putString(SettingConstants.COOKIE, cookieModel.getCookie()).apply();
                    Utils.setupCookies(cookieModel.getCookie());
                    cookieChanged = true;
                }
                dismiss();
            }
        };

        final View.OnLongClickListener onLongClickListener = v -> {
            final Object tag = v.getTag();

            if (tag instanceof DataBox.FavoriteModel) {
                final DataBox.FavoriteModel favoriteModel = (DataBox.FavoriteModel) tag;

                new AlertDialog.Builder(activity).setPositiveButton(R.string.yes, (d, which) -> Utils.dataBox.delFavorite(favoriteModel))
                        .setNegativeButton(R.string.no, null).setMessage(getString(R.string.quick_access_confirm_delete,
                        favoriteModel.getQuery())).show();

            } else if (tag instanceof DataBox.CookieModel) {
                final DataBox.CookieModel cookieModel = (DataBox.CookieModel) tag;

                if (cookieModel.isSelected())
                    Toast.makeText(context, R.string.quick_access_cannot_delete_curr, Toast.LENGTH_SHORT).show();
                else
                    new AlertDialog.Builder(activity).setPositiveButton(R.string.yes, (d, which) -> Utils.dataBox.delUserCookie(cookieModel))
                            .setNegativeButton(R.string.no, null).setMessage(getString(R.string.quick_access_confirm_delete,
                            cookieModel.getUsername())).show();
            }

            return true;
        };

        favoritesAdapter = new SimpleAdapter<>(activity, Utils.dataBox.getAllFavorites(), onClickListener, onLongClickListener);

        btnFavorite.setOnClickListener(onClickListener);

        final RecyclerView rvFavorites = contentView.findViewById(R.id.rvFavorites);
        final RecyclerView rvQuickAccess = contentView.findViewById(R.id.rvQuickAccess);

        final DividerItemDecoration itemDecoration = new DividerItemDecoration(activity, DividerItemDecoration.VERTICAL);
        rvFavorites.addItemDecoration(itemDecoration);
        rvFavorites.setAdapter(favoritesAdapter);

        if (!Utils.isEmpty(sharedPreferences.getString(SettingConstants.COOKIE, null))) {
            rvQuickAccess.addItemDecoration(itemDecoration);
            final ArrayList<DataBox.CookieModel> allCookies = Utils.dataBox.getAllCookies();
            final String strCookie = sharedPreferences.getString(SettingConstants.COOKIE, null);
            if (!Utils.isEmpty(strCookie) && allCookies != null) {
                for (final DataBox.CookieModel cookie : allCookies) {
                    if (strCookie.equals(cookie.getCookie())) {
                        cookie.setSelected(true);
                        break;
                    }
                }
            }
            rvQuickAccess.setAdapter(new SimpleAdapter<>(activity, allCookies, onClickListener, onLongClickListener));
        } else {
            ((View) rvQuickAccess.getParent()).setVisibility(View.GONE);
        }

        dialog.setContentView(contentView);
        return dialog;
    }

    @Override
    public void onDismiss(@NonNull final DialogInterface dialog) {
        super.onDismiss(dialog);
        if (cookieChanged && activity != null) activity.recreate();
    }

    @Override
    public void onShow(final DialogInterface dialog) {
        if (sharedPreferences.getBoolean(SettingConstants.SHOW_QUICK_ACCESS_DIALOG, true))
            new AlertDialog.Builder(activity).setMessage(R.string.quick_access_info_dialog)
                    .setPositiveButton(R.string.ok, null).setNeutralButton(R.string.dont_show_again, (d, which) ->
                    sharedPreferences.edit().putBoolean(SettingConstants.SHOW_QUICK_ACCESS_DIALOG, false).apply()).show();
    }
}