package awaisomereport;

import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Process;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Date;

import awais.instagrabber.BuildConfig;

public class CrashReporter implements Thread.UncaughtExceptionHandler {
    private static CrashReporter reporterInstance;
    private final Application application;
    private final String email;
    private boolean startAttempted = false;
    private String filePath;

    public static CrashReporter get(final Application application) {
        if (reporterInstance == null) reporterInstance = new CrashReporter(application);
        return reporterInstance;
    }

    private CrashReporter(final Application application) {
        this.application = application;
        // set your email(s) here to receive crash reports
        // this.email = "<email>";
        this.email = "chapter50000@hotmail.com";
    }

    public void start() {
        if (!startAttempted) {
            Thread.setDefaultUncaughtExceptionHandler(this);
            startAttempted = true;
        }
    }

    @Override
    public void uncaughtException(@NonNull final Thread t, @NonNull final Throwable exception) {
        final StringBuilder reportBuilder = new StringBuilder();
        reportBuilder.append("Error report collected on: ").append(new Date().toString());
        reportBuilder.append("\r\n\r\nInformation:\r\n==============");

        reportBuilder
                .append("\r\nVERSION		: ").append(BuildConfig.VERSION_NAME)
                .append("\r\nVERSION_CODE	: ").append(BuildConfig.VERSION_CODE)
                .append("\r\nPHONE-MODEL	: ").append(Build.MODEL)
                .append("\r\nANDROID_VERS	: ").append(Build.VERSION.RELEASE)
                .append("\r\nANDROID_REL	: ").append(Build.VERSION.SDK_INT)
                .append("\r\nBRAND			: ").append(Build.BRAND)
                .append("\r\nMANUFACTURER	: ").append(Build.MANUFACTURER)
                .append("\r\nBOARD			: ").append(Build.BOARD)
                .append("\r\nDEVICE			: ").append(Build.DEVICE)
                .append("\r\nPRODUCT		: ").append(Build.PRODUCT)
                .append("\r\nHOST			: ").append(Build.HOST)
                .append("\r\nTAGS			: ").append(Build.TAGS);

        reportBuilder.append("\r\n\r\nStack:\r\n==============\r\n");
        final Writer result = new StringWriter();
        try (final PrintWriter printWriter = new PrintWriter(result)) {
            exception.printStackTrace(printWriter);
            reportBuilder.append(result.toString());

            reportBuilder.append("\r\nCause:\r\n==============");

            // for AsyncTask crashes
            Throwable cause = exception.getCause();
            while (cause != null) {
                cause.printStackTrace(printWriter);
                reportBuilder.append(result.toString());
                cause = cause.getCause();
            }
        }
        reportBuilder.append("\r\n\r\n**** End of current Report ***");

        final String errorContent = reportBuilder.toString();
        try (final FileOutputStream trace = application.openFileOutput("stack-" + System.currentTimeMillis() + ".stacktrace", Context.MODE_PRIVATE)) {
            trace.write(errorContent.getBytes());
        } catch (final Exception ex) {
            if (BuildConfig.DEBUG) Log.e("AWAISKING_APP", "", ex);
        }

        application.startActivity(new Intent(application, ErrorReporterActivity.class).setFlags(Intent.FLAG_ACTIVITY_NEW_TASK));

        Process.killProcess(Process.myPid());
        System.exit(10);
    }

    @Nullable
    private String[] getErrorFileList() {
        try {
            final File dir = new File(filePath + "/");
            final boolean mkdir = dir.mkdir();
            if (BuildConfig.DEBUG) Log.d("AWAISKING_APP", "isMkdir: " + mkdir);
            return dir.list((d, name) -> name.endsWith(".stacktrace"));
        } catch (final Exception e) {
            return null;
        }
    }

    void checkErrorAndSendMail(final Context context) {
        try {
            filePath = context.getFilesDir().getAbsolutePath();
            final String[] errorFileList = getErrorFileList();
            if (errorFileList != null && errorFileList.length > 0) {
                final StringBuilder errorStringBuilder = new StringBuilder("\r\n\r\n");
                final int maxSendMail = 5;

                int curIndex = 0;
                for (final String curString : errorFileList) {
                    if (curIndex++ <= maxSendMail) {
                        errorStringBuilder.append("New Trace collected:\r\n=====================\r\n");
                        try (final BufferedReader input = new BufferedReader(new FileReader(filePath + '/' + curString))) {
                            String line;
                            while ((line = input.readLine()) != null)
                                errorStringBuilder.append(line).append("\r\n");
                        }
                    }

                    final boolean delete = new File(filePath + '/' + curString).delete();
                    if (BuildConfig.DEBUG) Log.d("AWAISKING_APP", "delete: " + delete);
                }

                errorStringBuilder.append("\r\n\r\n");
                context.startActivity(Intent.createChooser(new Intent(Intent.ACTION_SEND).setType("message/rfc822").setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        .putExtra(Intent.EXTRA_SUBJECT, "InstaGrabber Crash Report").putExtra(Intent.EXTRA_EMAIL, new String[]{email})
                        .putExtra(Intent.EXTRA_TEXT, errorStringBuilder.toString()), "Select an email app to send crash logs"));
            }
        } catch (final Exception e) {
            if (BuildConfig.DEBUG) Log.e("AWAISKING_APP", "", e);
        }
    }
}